/*
 *
 *
 *    stamps.cc
 *
 *    C++ Source Code for timestamp-generating functions.
 *
 *    Copyright (C) 2002,2003 by Matt Roberts, KK5JY,
 *    All rights reserved.
 *
 *    TODO (wishlist):
 *        1.  Tie the selection of 'time_func' to the option given in 'daylog'.
 *        2.  Fix TAI64N stamp function.
 *
 *    ---------------- L I C E N S E    S T A T E M E N T -------------------
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *    ----------------------- E N D    L I C E N S E ------------------------
 *
 *
 */


#include <sys/time.h>
#include <sys/timeb.h>
#include <string.h>
#include <string>

#include <stdio.h>

#include "stamps.h"

// TAI time when Unix time was = 0
#define TAI_UNIX_INTERSECTION (4611686018427387914ULL)

// FIXME: link this to the option in the main program
static struct tm *(*time_func)(const time_t *t) = localtime;

// formatting function
static std::string strftime(const std::string &format) {
	char buf[64];
	time_t now = time(NULL);
	strftime(buf, sizeof(buf), format.c_str(), time_func(&now));
	return std::string(buf);
}


/*
 *
 *   ctime_stamp()
 *
 */

std::string ctime_stamp (const char * const) {
	time_t now = time(NULL);
	std::string result(ctime(&now));
	if (*(result.end() - 1) == '\n')
		result.erase(result.end() - 1);
	return result;
}


/*
 *
 *   ftime_stamp()
 *
 */

std::string ftime_stamp (const char * const) {
	char buf[64];
	timeb ft;

	ftime (&ft);
	snprintf(buf, sizeof(buf), "%ld.%03d", ft.time, ft.millitm);
	return std::string(buf);
}


/*
 *
 *   time_stamp()
 *
 */

std::string time_stamp (const char * const) {
	char buf[64];
	time_t now;

	time(&now);
	snprintf(buf, sizeof(buf), "%ld", now);
	return std::string(buf);
}


/*
 *
 *   tai64n_stamp()
 *
 *   This code was stolen, um, borrowed from DJB's daemontools: http://cr.yp.to/
 *   I adapted it to work here, but the main idea came from Dan.
 *
 */

std::string tai64n_stamp (const char * const) {
	char buf[32];
	struct timeval now;

	// get the time
	gettimeofday(&now,(struct timezone *) 0);

	// seconds = now.tv_sec
	// microsec = 1000 * now.tv_usec + 500;

	// write it out the way we want
	snprintf(buf, sizeof(buf),
			"@%016llx""%08lx",
			(unsigned long long)((unsigned long long)now.tv_sec + TAI_UNIX_INTERSECTION),
			(unsigned long)(1000 * now.tv_usec + 500));

	// return the result
	return std::string(buf);
}


/*
 *
 *   rfc822_stamp()
 *
 */

std::string rfc2822_stamp (const char * const) {
	// i.e., 'Fri, 25 Apr 2003 22:36:53 -0500'
	return strftime("%a, %d %b %Y %H:%M:%S %z");
}

// EOF
